/*
 * Copyright 2004,2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
#include <woden/wsdl20/xml/axis2_woden_wsdl_element.h>
#include <woden/wsdl20/extensions/axis2_woden_element_extensible.h>
#include <woden/wsdl20/extensions/axis2_woden_attr_extensible.h>
#include <woden/wsdl20/extensions/axis2_woden_ext_element.h>
#include <woden/xml/axis2_woden_xml_attr.h>
#include <axis2_url.h>
#include <axis2_hash.h>

typedef struct axis2_woden_wsdl_element_impl axis2_woden_wsdl_element_impl_t;

/** 
 * @brief Wsdl Element Struct Impl
 *	Axis2 Wsdl Element  
 */ 
struct axis2_woden_wsdl_element_impl
{
    axis2_woden_wsdl_element_t wsdl_element;
    axis2_woden_obj_types_t obj_type;
    axis2_hash_t *methods;
    axis2_woden_attr_extensible_t *f_attr_ext;
    axis2_woden_element_extensible_t *f_elem_ext;
};

#define INTF_TO_IMPL(wsdl_element) \
    ((axis2_woden_wsdl_element_impl_t *) wsdl_element)

axis2_status_t AXIS2_CALL 
axis2_woden_wsdl_element_free(
        void *wsdl_element,
        axis2_env_t **envv);

axis2_woden_obj_types_t AXIS2_CALL 
axis2_woden_wsdl_element_type(
        void *wsdl_element,
        axis2_env_t **envv);

axis2_status_t AXIS2_CALL 
axis2_woden_wsdl_element_set_ext_attr(
        void *wsdl_element,
        axis2_env_t **env,
        axis2_qname_t *attr_type,
        axis2_woden_xml_attr_t *attr); 

void *AXIS2_CALL 
axis2_woden_wsdl_element_get_ext_attr(
        void *wsdl_element,
        axis2_env_t **env,
        axis2_qname_t *attr_type); 

axis2_array_list_t *AXIS2_CALL 
axis2_woden_wsdl_element_get_ext_attrs(
        void *wsdl_element,
        axis2_env_t **env); 

axis2_array_list_t *AXIS2_CALL 
axis2_woden_wsdl_element_get_ext_attrs_for_namespace(
        void *wsdl_element,
        axis2_env_t **env,
        axis2_url_t *namespc);

axis2_bool_t AXIS2_CALL 
axis2_woden_wsdl_element_has_ext_attrs_for_namespace(
        void *wsdl_element,
        axis2_env_t **env,
        axis2_url_t *namespc);

axis2_status_t AXIS2_CALL 
axis2_woden_wsdl_element_add_ext_element(
        void *wsdl_element,
        axis2_env_t **env,
        axis2_woden_ext_element_t *ext_el); 

axis2_status_t AXIS2_CALL 
axis2_woden_wsdl_element_remove_ext_element(
        void *wsdl_element,
        axis2_env_t **env,
        axis2_woden_ext_element_t *ext_el); 

axis2_array_list_t *AXIS2_CALL 
axis2_woden_wsdl_element_get_ext_elements(
        void *wsdl_element,
        axis2_env_t **env); 

axis2_array_list_t *AXIS2_CALL 
axis2_woden_wsdl_element_get_ext_elements_of_type(
        void *wsdl_element,
        axis2_env_t **env,
        axis2_qname_t *elem_type);

axis2_bool_t AXIS2_CALL 
axis2_woden_wsdl_element_has_ext_elements_for_namespace(
        void *wsdl_element,
        axis2_env_t **env,
        axis2_url_t *namespc);

axis2_status_t AXIS2_CALL 
axis2_woden_wsdl_element_add_ext_element(
        void *wsdl_element,
        axis2_env_t **env,
axis2_woden_ext_element_t *ext_el); 

axis2_status_t AXIS2_CALL 
axis2_woden_wsdl_element_remove_ext_element(
        void *wsdl_element,
        axis2_env_t **env,
        axis2_woden_ext_element_t *ext_el);

axis2_array_list_t *AXIS2_CALL 
axis2_woden_wsdl_element_get_ext_elements(
        void *wsdl_element,
        axis2_env_t **env); 

axis2_array_list_t *AXIS2_CALL 
axis2_woden_wsdl_element_get_ext_elements_of_type(
        void *wsdl_element,
        axis2_env_t **env,
        axis2_qname_t *ext_type);

axis2_bool_t AXIS2_CALL 
axis2_woden_wsdl_element_has_ext_elements_for_namespace(
        void *wsdl_element,
        axis2_env_t **env,
        axis2_url_t *namespc);

static axis2_woden_wsdl_element_t *
create(
        axis2_env_t **env);

/************************Woden C Internal Methods******************************/
AXIS2_DECLARE(axis2_woden_wsdl_element_t *)
axis2_woden_wsdl_element_to_attr_extensible(
        void *wsdl_element,
        axis2_env_t **env)
{
    axis2_woden_wsdl_element_impl_t *wsdl_element_impl = NULL;

    AXIS2_ENV_CHECK(env, AXIS2_FAILURE);

    if(!wsdl_element)
    {
        wsdl_element_impl = (axis2_woden_wsdl_element_impl_t *) create(env);
    }
    else
        wsdl_element_impl = (axis2_woden_wsdl_element_impl_t *) wsdl_element;
    wsdl_element_impl->wsdl_element.base.attr_extensible.ops = 
            AXIS2_MALLOC((*env)->allocator, 
            sizeof(axis2_woden_attr_extensible_ops_t));
    axis2_woden_attr_extensible_resolve_methods(&(wsdl_element_impl->
            wsdl_element.base.attr_extensible), env, wsdl_element_impl->methods);
    return wsdl_element;

}

axis2_status_t AXIS2_CALL
axis2_woden_wsdl_element_to_attr_extensible_free(
        void *wsdl_element,
        axis2_env_t **env)
{
    axis2_woden_wsdl_element_impl_t *wsdl_element_impl = NULL;

    AXIS2_ENV_CHECK(env, AXIS2_FAILURE);
    wsdl_element_impl = INTF_TO_IMPL(wsdl_element);

    if(wsdl_element_impl->wsdl_element.base.attr_extensible.ops)
    {
        AXIS2_FREE((*env)->allocator, wsdl_element_impl->wsdl_element.base.
                attr_extensible.ops);
        wsdl_element_impl->wsdl_element.base.attr_extensible.ops = NULL;
    }
    return AXIS2_SUCCESS;
}

AXIS2_DECLARE(axis2_woden_wsdl_element_t *)
axis2_woden_wsdl_element_to_element_extensible(
        void *wsdl_element,
        axis2_env_t **env)
{
    axis2_woden_wsdl_element_impl_t *wsdl_element_impl = NULL;

    AXIS2_ENV_CHECK(env, AXIS2_FAILURE);

    if(!wsdl_element)
    {
        wsdl_element_impl = (axis2_woden_wsdl_element_impl_t *) create(env);
    }
    else
        wsdl_element_impl = (axis2_woden_wsdl_element_impl_t *) wsdl_element;
    wsdl_element_impl->wsdl_element.base.element_extensible.ops = 
            AXIS2_MALLOC((*env)->allocator, 
            sizeof(axis2_woden_element_extensible_ops_t));
    axis2_woden_element_extensible_resolve_methods(&(wsdl_element_impl->
            wsdl_element.base.element_extensible), env, wsdl_element_impl->methods);
    return wsdl_element;

}

axis2_status_t AXIS2_CALL
axis2_woden_wsdl_element_to_element_extensible_free(
        void *wsdl_element,
        axis2_env_t **env)
{
    axis2_woden_wsdl_element_impl_t *wsdl_element_impl = NULL;

    AXIS2_ENV_CHECK(env, AXIS2_FAILURE);
    wsdl_element_impl = INTF_TO_IMPL(wsdl_element);

    if(wsdl_element_impl->wsdl_element.base.element_extensible.ops)
    {
        AXIS2_FREE((*env)->allocator, wsdl_element_impl->wsdl_element.base.
                element_extensible.ops);
        wsdl_element_impl->wsdl_element.base.element_extensible.ops = NULL;
    }
    return AXIS2_SUCCESS;
}


/************************End of Woden C Internal Methods***********************/
static axis2_woden_wsdl_element_t *
create(
        axis2_env_t **env)
{
    axis2_woden_wsdl_element_impl_t *wsdl_element_impl = NULL;
    
    AXIS2_ENV_CHECK(env, AXIS2_FAILURE);
    wsdl_element_impl = AXIS2_MALLOC((*env)->allocator, 
                    sizeof(axis2_woden_wsdl_element_impl_t));

    wsdl_element_impl->methods = NULL;
    wsdl_element_impl->obj_type = AXIS2_WODEN_WSDL_ELEMENT;
    wsdl_element_impl->f_attr_ext = NULL;
    wsdl_element_impl->f_elem_ext = NULL;

    wsdl_element_impl->wsdl_element.base.attr_extensible.ops = NULL;
    wsdl_element_impl->wsdl_element.base.element_extensible.ops = NULL;
    wsdl_element_impl->wsdl_element.ops = 
            AXIS2_MALLOC((*env)->allocator, 
            sizeof(axis2_woden_wsdl_element_ops_t)); 
   
    wsdl_element_impl->wsdl_element.ops->free = axis2_woden_wsdl_element_free;
    wsdl_element_impl->wsdl_element.ops->type = axis2_woden_wsdl_element_type;
    
    wsdl_element_impl->methods = axis2_hash_make(env);
    if(!wsdl_element_impl->methods) 
    {
        AXIS2_ERROR_SET((*env)->error, AXIS2_ERROR_NO_MEMORY, AXIS2_FAILURE);
        return NULL;
    }
    axis2_hash_set(wsdl_element_impl->methods, "free", 
            AXIS2_HASH_KEY_STRING, axis2_woden_wsdl_element_free);
    axis2_hash_set(wsdl_element_impl->methods, "to_attr_extensible_free", 
            AXIS2_HASH_KEY_STRING, 
            axis2_woden_wsdl_element_to_attr_extensible_free);
    axis2_hash_set(wsdl_element_impl->methods, "to_element_extensible_free", 
            AXIS2_HASH_KEY_STRING, 
            axis2_woden_wsdl_element_to_element_extensible_free);
    axis2_hash_set(wsdl_element_impl->methods, "type", 
            AXIS2_HASH_KEY_STRING, axis2_woden_wsdl_element_type);
    axis2_hash_set(wsdl_element_impl->methods, "set_ext_attr", 
            AXIS2_HASH_KEY_STRING, axis2_woden_wsdl_element_set_ext_attr);
    axis2_hash_set(wsdl_element_impl->methods, "get_ext_attr", 
            AXIS2_HASH_KEY_STRING, axis2_woden_wsdl_element_get_ext_attr);
    axis2_hash_set(wsdl_element_impl->methods, "get_ext_attrs", 
            AXIS2_HASH_KEY_STRING, axis2_woden_wsdl_element_get_ext_attrs);
    axis2_hash_set(wsdl_element_impl->methods, "get_ext_attrs_for_namespace", 
            AXIS2_HASH_KEY_STRING, 
            axis2_woden_wsdl_element_get_ext_attrs_for_namespace);
    axis2_hash_set(wsdl_element_impl->methods, "has_ext_attrs_for_namespace", 
            AXIS2_HASH_KEY_STRING, 
            axis2_woden_wsdl_element_has_ext_attrs_for_namespace);
    axis2_hash_set(wsdl_element_impl->methods, "add_ext_element", 
            AXIS2_HASH_KEY_STRING, axis2_woden_wsdl_element_add_ext_element);
    axis2_hash_set(wsdl_element_impl->methods, "remove_ext_element", 
            AXIS2_HASH_KEY_STRING, axis2_woden_wsdl_element_remove_ext_element);
    axis2_hash_set(wsdl_element_impl->methods, "get_ext_elements", 
            AXIS2_HASH_KEY_STRING, axis2_woden_wsdl_element_get_ext_elements);
    axis2_hash_set(wsdl_element_impl->methods, "get_ext_elements_of_type", 
            AXIS2_HASH_KEY_STRING, 
            axis2_woden_wsdl_element_get_ext_elements_of_type);
    axis2_hash_set(wsdl_element_impl->methods, "has_ext_elements_for_namespace", 
            AXIS2_HASH_KEY_STRING, 
            axis2_woden_wsdl_element_has_ext_elements_for_namespace);
    
    return &(wsdl_element_impl->wsdl_element);
}

AXIS2_DECLARE(axis2_woden_wsdl_element_t *)
axis2_woden_wsdl_element_create(
        axis2_env_t **env)
{
    axis2_woden_wsdl_element_impl_t *wsdl_element_impl = NULL;
    
    AXIS2_ENV_CHECK(env, AXIS2_FAILURE);
    wsdl_element_impl = (axis2_woden_wsdl_element_impl_t *) create;

    wsdl_element_impl->f_attr_ext = axis2_woden_attr_extensible_create(env);
    if(!wsdl_element_impl->f_attr_ext)
    {
        return NULL;
    }
    wsdl_element_impl->f_elem_ext = axis2_woden_element_extensible_create(env);
    if(!wsdl_element_impl->f_elem_ext)
    {
        return NULL;
    }

    return &(wsdl_element_impl->wsdl_element);
}

axis2_status_t AXIS2_CALL
axis2_woden_wsdl_element_free(
        void *wsdl_element,
        axis2_env_t **env)
{
    axis2_woden_wsdl_element_impl_t *wsdl_element_impl = NULL;

    AXIS2_ENV_CHECK(env, AXIS2_FAILURE);
    wsdl_element_impl = INTF_TO_IMPL(wsdl_element);

    if(wsdl_element_impl->f_attr_ext)
    {
        AXIS2_WODEN_ATTR_EXTENSIBLE_FREE(wsdl_element_impl->f_attr_ext, env);
        wsdl_element_impl->f_attr_ext = NULL;
    }
    
    if(wsdl_element_impl->f_elem_ext)
    {
        AXIS2_WODEN_ELEMENT_EXTENSIBLE_FREE(wsdl_element_impl->f_elem_ext, env);
        wsdl_element_impl->f_elem_ext = NULL;
    }

    if(wsdl_element_impl->wsdl_element.base.attr_extensible.ops)
    {
        AXIS2_FREE((*env)->allocator, wsdl_element_impl->wsdl_element.base.
                attr_extensible.ops);
        wsdl_element_impl->wsdl_element.base.attr_extensible.ops = NULL;
    }

    if(wsdl_element_impl->wsdl_element.base.element_extensible.ops)
    {
        AXIS2_FREE((*env)->allocator, wsdl_element_impl->wsdl_element.base.
                element_extensible.ops);
        wsdl_element_impl->wsdl_element.base.element_extensible.ops = NULL;
    }

    if(wsdl_element_impl->wsdl_element.ops)
    {
        AXIS2_FREE((*env)->allocator, wsdl_element_impl->wsdl_element.ops);
        wsdl_element_impl->wsdl_element.ops = NULL;
    }

    if(wsdl_element_impl)
    {
        AXIS2_FREE((*env)->allocator, wsdl_element_impl);
        wsdl_element_impl = NULL;
    }
    return AXIS2_SUCCESS;
}

axis2_woden_obj_types_t AXIS2_CALL
axis2_woden_wsdl_element_type(
        void *wsdl_element,
        axis2_env_t **env)
{
    axis2_woden_wsdl_element_impl_t *wsdl_element_impl = NULL;

    AXIS2_ENV_CHECK(env, AXIS2_FAILURE);
    wsdl_element_impl = INTF_TO_IMPL(wsdl_element);
    
    return wsdl_element_impl->obj_type;
}

axis2_status_t AXIS2_CALL
axis2_woden_wsdl_element_resolve_methods(
        axis2_woden_wsdl_element_t *wsdl_element,
        axis2_env_t **env,
        axis2_hash_t *methods)
{
    AXIS2_ENV_CHECK(env, AXIS2_FAILURE);
    AXIS2_PARAM_CHECK((*env)->error, methods, AXIS2_FAILURE);
    
    wsdl_element->ops = AXIS2_MALLOC((*env)->allocator, 
            sizeof(axis2_woden_wsdl_element_ops_t));
    wsdl_element->ops->free = axis2_hash_get(methods, "free", 
            AXIS2_HASH_KEY_STRING);
    wsdl_element->ops->to_wsdl_element_free = axis2_hash_get(methods, 
            "to_wsdl_element_free", AXIS2_HASH_KEY_STRING);
    wsdl_element->ops->type = axis2_hash_get(methods, "type", 
            AXIS2_HASH_KEY_STRING);

    return AXIS2_SUCCESS;    
}

axis2_status_t AXIS2_CALL 
axis2_woden_wsdl_element_set_ext_attr(
        void *wsdl_element,
        axis2_env_t **env,
        axis2_qname_t *attr_type,
        axis2_woden_xml_attr_t *attr)
{
    axis2_woden_wsdl_element_impl_t *wsdl_element_impl = NULL;

    AXIS2_ENV_CHECK(env, AXIS2_FAILURE);
    AXIS2_PARAM_CHECK((*env)->error, attr_type, AXIS2_FAILURE);
    AXIS2_PARAM_CHECK((*env)->error, attr, AXIS2_FAILURE);
    wsdl_element_impl = INTF_TO_IMPL(wsdl_element);
    return AXIS2_WODEN_ATTR_EXTENSIBLE_SET_EXT_ATTR(wsdl_element_impl->f_attr_ext, 
            env, attr_type, attr);    
}

void *AXIS2_CALL 
axis2_woden_wsdl_element_get_ext_attr(
        void *wsdl_element,
        axis2_env_t **env,
        axis2_qname_t *attr_type) 
{
    axis2_woden_wsdl_element_impl_t *wsdl_element_impl = NULL;

    AXIS2_ENV_CHECK(env, AXIS2_FAILURE);
    AXIS2_PARAM_CHECK((*env)->error, attr_type, AXIS2_FAILURE);
    wsdl_element_impl = INTF_TO_IMPL(wsdl_element);
    
    return AXIS2_WODEN_ATTR_EXTENSIBLE_GET_EXT_ATTR(wsdl_element_impl->f_attr_ext, 
            env, attr_type);
}

axis2_array_list_t *AXIS2_CALL 
axis2_woden_wsdl_element_get_ext_attrs(
        void *wsdl_element,
        axis2_env_t **env) 
{
    axis2_woden_wsdl_element_impl_t *wsdl_element_impl = NULL;
    
    AXIS2_ENV_CHECK(env, AXIS2_FAILURE);
    wsdl_element_impl = INTF_TO_IMPL(wsdl_element);

    return AXIS2_WODEN_ATTR_EXTENSIBLE_GET_EXT_ATTRS(wsdl_element_impl->f_attr_ext, 
            env);
}

axis2_array_list_t *AXIS2_CALL 
axis2_woden_wsdl_element_get_ext_attrs_for_namespace(
        void *wsdl_element,
        axis2_env_t **env,
        axis2_url_t *namespc) 
{
    axis2_woden_wsdl_element_impl_t *wsdl_element_impl = NULL;
    
    AXIS2_ENV_CHECK(env, AXIS2_FAILURE);
    AXIS2_PARAM_CHECK((*env)->error, namespc, AXIS2_FAILURE);
    wsdl_element_impl = INTF_TO_IMPL(wsdl_element);

    return AXIS2_WODEN_ATTR_EXTENSIBLE_GET_EXT_ATTRS_FOR_NAMESPACE(
            wsdl_element_impl->f_attr_ext, env, namespc);
}

axis2_bool_t AXIS2_CALL 
axis2_woden_wsdl_element_has_ext_attrs_for_namespace(
        void *wsdl_element,
        axis2_env_t **env,
        axis2_url_t *namespc)
{
    axis2_woden_wsdl_element_impl_t *wsdl_element_impl = NULL;

    AXIS2_ENV_CHECK(env, AXIS2_FAILURE);
    AXIS2_PARAM_CHECK((*env)->error, namespc, AXIS2_FAILURE);
    wsdl_element_impl = INTF_TO_IMPL(wsdl_element);

    return AXIS2_WODEN_ATTR_EXTENSIBLE_HAS_EXT_ATTRS_FOR_NAMESPACE(
            wsdl_element_impl->f_attr_ext, env, namespc);
}

axis2_status_t AXIS2_CALL 
axis2_woden_wsdl_element_add_ext_element(
        void *wsdl_element,
        axis2_env_t **env,
axis2_woden_ext_element_t *ext_el) 
{
    axis2_woden_wsdl_element_impl_t *wsdl_element_impl = NULL;

    AXIS2_ENV_CHECK(env, AXIS2_FAILURE);
    AXIS2_PARAM_CHECK((*env)->error, ext_el, AXIS2_FAILURE);
    wsdl_element_impl = INTF_TO_IMPL(wsdl_element);
    
    return AXIS2_WODEN_ELEMENT_EXTENSIBLE_ADD_EXT_ELEMENT(wsdl_element_impl->
            f_elem_ext, env, ext_el);
}

axis2_status_t AXIS2_CALL 
axis2_woden_wsdl_element_remove_ext_element(
        void *wsdl_element,
        axis2_env_t **env,
axis2_woden_ext_element_t *ext_el) 
{
    axis2_woden_wsdl_element_impl_t *wsdl_element_impl = NULL;

    AXIS2_ENV_CHECK(env, AXIS2_FAILURE);
    AXIS2_PARAM_CHECK((*env)->error, ext_el, AXIS2_FAILURE);
    wsdl_element_impl = INTF_TO_IMPL(wsdl_element);

    return AXIS2_WODEN_ELEMENT_EXTENSIBLE_REMOVE_EXT_ELEMENT(wsdl_element_impl->
            f_elem_ext, env, ext_el);
}

axis2_array_list_t *AXIS2_CALL 
axis2_woden_wsdl_element_get_ext_elements(
        void *wsdl_element,
axis2_env_t **env) 
{
    axis2_woden_wsdl_element_impl_t *wsdl_element_impl = NULL;

    AXIS2_ENV_CHECK(env, AXIS2_FAILURE);
    wsdl_element_impl = INTF_TO_IMPL(wsdl_element);
    
    return AXIS2_WODEN_ELEMENT_EXTENSIBLE_GET_EXT_ELEMENTS(wsdl_element_impl->
            f_elem_ext, env);
}

axis2_array_list_t *AXIS2_CALL 
axis2_woden_wsdl_element_get_ext_elements_of_type(
        void *wsdl_element,
        axis2_env_t **env,
axis2_qname_t *ext_type) 
{
    axis2_woden_wsdl_element_impl_t *wsdl_element_impl = NULL;
    
    AXIS2_ENV_CHECK(env, AXIS2_FAILURE);
    AXIS2_PARAM_CHECK((*env)->error, ext_type, AXIS2_FAILURE);
    wsdl_element_impl = INTF_TO_IMPL(wsdl_element);

    return AXIS2_WODEN_ELEMENT_EXTENSIBLE_GET_EXT_ELEMENTS_OF_TYPE(
            wsdl_element_impl->f_elem_ext, env, ext_type);
}

axis2_bool_t AXIS2_CALL 
axis2_woden_wsdl_element_has_ext_elements_for_namespace(
        void *wsdl_element,
        axis2_env_t **env,
        axis2_url_t *namespc)
{
    axis2_woden_wsdl_element_impl_t *wsdl_element_impl = NULL;

    AXIS2_ENV_CHECK(env, AXIS2_FAILURE);
    AXIS2_PARAM_CHECK((*env)->error, namespc, AXIS2_FAILURE);
    wsdl_element_impl = INTF_TO_IMPL(wsdl_element);

    return AXIS2_WODEN_ELEMENT_EXTENSIBLE_HAS_EXT_ELEMENTS_FOR_NAMESPACE(
            wsdl_element_impl->f_elem_ext, env, namespc);
}


